library jaguar_mux.example.simple.server;

import 'package:jaguar/jaguar.dart';
import 'package:jaguar_mux/jaguar_mux.dart';
import 'package:jaguar/interceptors.dart';

class WrapSimpleInterceptor extends RouteWrapper<SimpleInterceptor> {
  final String name;

  const WrapSimpleInterceptor(this.name);

  SimpleInterceptor createInterceptor() => new SimpleInterceptor(name);
}

class SimpleInterceptor extends Interceptor {
  final String name;

  SimpleInterceptor(this.name);

  String pre() => 'input';

  void post() => print('post $name');
}

Map versionRoute() => {'v': 0.1};

int getUserById(int id) => id;

Map getQueryParams(
    @Input(SimpleInterceptor) String input, @InputHeader('hello') String hello,
    {String name, String password}) {
  return {
    'n': name,
    'p': password,
    'h': hello,
  };
}

Map decodeJsonMap(/* Request req, */ @Input(DecodeJsonMap) Map body) {
  return body;
}

main() async {
  final muxer = new MuxBuilder();

  muxer
      .get('/encode/maptojson', () => {'name': 'jaguar'})
      .wrap(new WrapEncodeMapToJson());

  muxer
      .get('/wrap/simpleinterceptor', () => {'name': 'jaguar'})
      .wrap(new WrapSimpleInterceptor('hello'));

  muxer.get('/user/:id', getUserById);

  muxer.get('/version', versionRoute);

  muxer
      .get('/query/params', getQueryParams)
      .wrap(new WrapSimpleInterceptor('hello'));

  muxer.post('/decode/json/map', decodeJsonMap).wrap(new WrapDecodeJsonMap());

  Mux mux = muxer.build();

  Jaguar configuration = new Jaguar();
  configuration.addApi(mux);

  await configuration.serve();
}
